@@ -272,7 +272,7 @@ class ABNF(object):
                            | self.opcode)
         if length < ABNF.LENGTH_7:
             frame_header += chr(self.mask << 7 | length)
-        elif length <= ABNF.LENGTH_16:
+        elif length < ABNF.LENGTH_16:
             frame_header += chr(self.mask << 7 | 0x7e)
             frame_header += struct.pack("!H", length)
         else:
@@ -554,7 +554,6 @@ class WebSocket(object):
         rsv2 = b1 >> 5 & 1
         rsv3 = b1 >> 4 & 1
         opcode = b1 & 0xf
-        
         b2 = ord(header_bytes[1])
         mask = b2 >> 7 & 1
         length = b2 & 0x7f
@@ -570,7 +569,7 @@ class WebSocket(object):
         mask_key = ""
         if mask:
             mask_key = self._recv(4)
-        data = self._recv(length)
+        data = self._recv_strict(length)
         if traceEnabled:
             recieved = header_bytes + length_data + mask_key + data
             logger.debug("recv: " + repr(recieved))
@@ -589,7 +588,7 @@ class WebSocket(object):
 
         reason: the reason to close. This must be string.
         """
-        if status < 0 or status > ABNF.LENGTH_16:
+        if status < 0 or status >= ABNF.LENGTH_16:
             raise ValueError("code is invalid range")
         self.send(struct.pack('!H', status) + reason, ABNF.OPCODE_CLOSE)
         
@@ -604,7 +603,7 @@ class WebSocket(object):
         reason: the reason to close. This must be string.
         """
         if self.connected:
-            if status < 0 or status > ABNF.LENGTH_16:
+            if status < 0 or status >= ABNF.LENGTH_16:
                 raise ValueError("code is invalid range")
 
             try:
